#include <iostream>
#include <cstdint>
#include <algorithm>
#include <cassert>

template <size_t N>
class SegmentTree {
private:
	int64_t sum[2 * N];
	uint32_t len[2 * N];
	int64_t lazyC[N];
	int64_t lazyP[N];

	uint32_t depth;

	inline static int64_t arith_sum(int64_t c, int64_t p, int64_t l) {
		int64_t ans =  (2 * c  + (l - 1) * p) * l / 2;
		return ans;
	}

	inline void reeval(uint32_t n) {
		sum[n] = sum[n * 2] + sum[n * 2 + 1];
		sum[n] += arith_sum(lazyC[n], lazyP[n], len[n]);
	}

	inline void update_val(uint32_t i, int64_t c, int64_t p) {
		sum[i] += arith_sum(c, p, len[i]);

		if (i < N) {
			lazyC[i] += c;
			lazyP[i] += p;
		}
	}

	inline void propagate(uint32_t i) {
		update_val(i * 2, lazyC[i], lazyP[i]);
		update_val(i * 2 + 1, lazyC[i] + lazyP[i] * len[i * 2], lazyP[i]);
		lazyP[i] = lazyC[i] = 0;
	}

	inline void push_lazy(uint32_t n) {
		for (uint32_t i = depth;i > 0;i--) {
			uint32_t c = n >> i;
			if (c == 0) {
				continue;
			}
			if ((lazyC[c] != 0) || (lazyP[c] != 0)) {
				propagate(c);
			}
		}
	}

	inline void push_updates(uint32_t n) {
		for (n >>= 1; n > 0;n >>= 1) {
			reeval(n);
		}
	}


public:
	SegmentTree() {
		//init(N);
	}
	void init(uint32_t n) {
		uint32_t c = 2 * n - 1;
		depth = 0;
		while (c > 1) {
			c /= 2;
			depth++;
		}

		std::fill(sum, sum + 2 * n, 0);
		std::fill(lazyC, lazyC + n, 0);
		std::fill(lazyP, lazyP + n, 0);

		std::fill(len, len + 2 * n, 1);
		for (uint32_t i = n - 1;i > 0;i--) {
			len[i] = len[i * 2] + len[i * 2 + 1];
		}
	}



	inline void update(uint32_t l, uint32_t r, int64_t c, int64_t p) {
		int64_t lC = c;
		int64_t rC = (r - l) * p + c;


		uint32_t oL = l += N;
		uint32_t oR = r += N;

		push_lazy(oL);
		push_lazy(oR - 1);

		for(;l < r;l >>= 1, r >>= 1) {
			if (l & 1) {
				update_val(l, lC, p);
				lC += len[l] * p;
				l++;
			}
			if (r & 1) {
				--r;
				rC -= len[r] * p;
				update_val(r, rC, p);
			}
		}

		push_updates(oL);
		push_updates(oR - 1);
	}

	inline int64_t query(uint32_t l, uint32_t r) {
		l += N;
		r += N;

		push_lazy(l);
		push_lazy(r - 1);

		//std::cerr << "ans l (" << l - N << ") = " << sum[l] << "\n";

		int64_t ans = 0;
		for(;l < r;l >>= 1, r >>= 1) {
			if (l & 1) {
				ans += sum[l++];
			}
			if (r & 1) {
				ans += sum[--r];
			}
		}

		return ans;
	}
};

const uint32_t MAX_N = 1e5 + 1;

SegmentTree<MAX_N> segTree;

int main() {
	std::ios::sync_with_stdio(false);
	std::cin.tie(nullptr);
	uint32_t n, q;
	std::cin >> n >> q;
	segTree.init(n);
	for (uint32_t i = 0;i < q;i++) {
		uint32_t t;
		std::cin >> t;
		if (t == 1) {
			uint32_t row, l, r;
			int64_t c, p;
			std::cin >> row >> l >> r >> c >> p;

			segTree.update(l - 1, r, c, p);
		} else if (t == 2) {
			uint32_t row, l, r;
			std::cin >> row >> l >> r;

			std::cout << segTree.query(l - 1, r) << "\n";
			std::cout << std::flush;
		}


	}
}
